/*****************************************************************************
**																			**
**	 Name: 	Init SDRAM using assembly language  							**
**																			**
******************************************************************************

(C) Copyright 2006 - Analog Devices, Inc.  All rights reserved.

Project Name:	Init SDRAM Example

Author: 		Fabian (DSP Application Engineer)

Version:		v1.0

Last Modified:	09/08/06

History:		v1.0 09/08/06

Software:		VisualDSP++ 5.0 

Hardware:		ADSP-BF537 EZ-KIT Lite

Connections:	ADDS-HPUSB-ICE

Settings:				

Purpose:		

Comments:		Run this program and take a lot into the Blackfin Memory


*****************************************************************************/
#include <defbf537.h>

/****************************************************************************
Main Program
*****************************************************************************/
.section L1_code;
.global _main;

_main:
	call _SetPLL; 	//Set the PLL to 600 cclk and 100sclk
	call _SetSDRAM; //Set the SDRAM 
	//access the SDRAM
	P0.L = lo(0x4);
	P0.H = hi(0x4);
	R0.H = 0xEFBE; //BEEF in little Endian
	R0.L = 0xADDE; //DEAD in little Endian
	[P0] = R0;

forever:
	jump forever;

_main.end:

/****************************************************************************
End of Main
*****************************************************************************/

/****************************************************************************
Functions
*****************************************************************************/


/*****************************************************************************
Function: _SetSDRAM
Purpose: Sets up the SDRAM
Input Parameters: none
Return Parameters: none
Registers Used: R0,P0
C-Callable : YES
*****************************************************************************/

_SetSDRAM:

//SDRAM Refresh Rate Setting 
    P0.H = hi(EBIU_SDRRC);
    P0.L = lo(EBIU_SDRRC);
    R0 = 0x307 (z);
    w[P0] = R0;

//SDRAM Memory Bank Control Register    
	P0.H = hi(EBIU_SDBCTL);
    P0.L = lo(EBIU_SDBCTL);
    R0 = 		EBCAW_10  | //Page size 1024 
				EBSZ_64   | //64 MB of SDRAM
				EBE; 		//SDRAM enable
	w[P0] = R0;
//SDRAM Memory Global Control Register
	P0.H = hi(EBIU_SDGCTL);
    P0.L = lo(EBIU_SDGCTL);	
    R0.H=hi(	~CDDBG	& // Control disable during bus grant off
				~TCSR	& // Temperature compensated self-refresh off 
				~EMREN 	& // Extended mode register enabled off 
				~FBBRW	& // Fast back to back read to write off 	
				~EBUFE	& // External buffering enabled off
				~SRFS	& // Self-refresh setting off
				~PSM	& // Powerup sequence mode (PSM) first
				~PUPSD	& // Powerup start delay (PUPSD) off
		    	PSS		| // Powerup sequence start enable (PSSE) on
				TWR_2	| // Write to precharge delay TWR = 2 (14-15 ns)
				TRCD_2	| // RAS to CAS delay TRCD =2 (15-20ns)
				TRP_2	| // Bank precharge delay TRP = 2 (15-20ns)
				TRAS_4	| // Bank activate command delay TRAS = 4
				PASR_ALL| // Partial array self refresh
				CL_3	| // CAS latency
				SCTLE)	; // SDRAM clock enable	
				
	R0.L= 	lo(~CDDBG	& // Control disable during bus grant off
				~TCSR	& // Temperature compensated self-refresh off 
				~EMREN 	& // Extended mode register enabled off 
				~FBBRW	& // Fast back to back read to write off 	
				~EBUFE	& // External buffering enabled off
				~SRFS	& // Self-refresh setting off
				~PSM	& // Powerup sequence mode (PSM) first
				~PUPSD	& // Powerup start delay (PUPSD) off
		    	PSS		| // Powerup sequence start enable (PSSE) on
				TWR_2	| // Write to precharge delay TWR = 2 (14-15 ns)
				TRCD_2	| // RAS to CAS delay TRCD =2 (15-20ns)
				TRP_2	| // Bank precharge delay TRP = 2 (15-20ns)
				TRAS_4	| // Bank activate command delay TRAS = 4
				PASR_ALL| // Partial array self refresh
				CL_3	| // CAS latency
				SCTLE)	; // SDRAM clock enable	
	[P0] = R0;					

	RTS;
_SETSDRAM.end:

/*****************************************************************************
Function: _SetPLL
Purpose: Sets up the clocks
Input Parameters: none
Return Parameters: none
Registers Used: R0,P0
C-Callable : YES
*****************************************************************************/
_SetPLL:  

    //Ensure no other interrupt will disturb us
    CLI R1;


    //Setup the DIV of ssclk and cclk
    P0.H = hi(PLL_DIV);
    P0.L = lo(PLL_DIV);
    R0 = 0x0006 (z);
    w[P0] = R0;

    //Setup wait cycles until PLL is set
    P0.L = lo(PLL_LOCKCNT);
    R0 = 0x200 (z);
    w[P0] = R0;

    //Setup the PLL
    P0.L = lo(PLL_CTL);
    R0 = 0x3000 (z);
    w[P0] = R0;

    //Wait the PLL settings are done
    idle;

    //restore interrupts
    STI R1;

    RTS;
_SetPLL.end:

